<?php
require_once('lassospkit_helper.inc.php');
require_once('lassospkit_debug.inc.php');
require_once('lassospkit_utils.inc.php');
require_once('lassospkit_url_dispatch.inc.php');
require_once('lassospkit_generic_session.inc.php');
require_once('lassospkit_dummysession.inc.php');
require_once('lassospkit_autopersistentsession.inc.php');

class LassoSPKitEndpoint extends LassoSPKitUrlDispatch {
    var $relayState = null;

    function LassoSPKitEndpoint() {
        $this->addDispatch('/assertionConsumer', 'assertionConsumer');
        $this->addDispatch('/sloBrws','sloBrws');
        $this->addDispatch('/sloSoap','sloSoap');
        $this->addDispatch('/sloReturn','sloReturn');
    }

    /** Return the session object. */
    function getSession() {
        static $session;
        if ($session == null) {
            $session_class = "LassoSPKit" . LassoSPKitConfig::get('session');
            $session = new $session_class();
        }
        return $session;
    }
    function distpatchAndExit() {
        if (isset($_SERVER['HTTP_REFERER'])) {
            $host = $_SERVER['HTTP_REFERER'];
            $this->verifyUrl($host);
        }
        parent::dispatchAndExit();
    }
    /** Verify that the host is the same has HTTP_HOST */
    function verifyUrl($host) {
        $host = strstr('//', $host);
        $pos = strpos($host, '/');
        if ($pos !== FALSE) {
            $host = substr($host, 0, $pos);
        }

        if ($host && isset($_SERVER['HTTP_HOST']) && $host != $_SERVER['HTTP_HOST']) {
            echo "Bad referer '$host' != '" . $_SERVER['HTTP_HOST'] . "'";
            exit(1);
        }
    }
    function getProfileObject() {
        throw new Exception('Not implemented');
    }
    function assertionConsumer() {
        try {
            $profile = $this->getProfileObject();
            $http_method = $this->limitMethodBrws($this->identifyHttpMethod());
            $query_string = $this->getQueryString($http_method);
            $ok = $profile->ssoConsumer($http_method, $query_string);
            $this->relayState = $profile->relayState;
            return $ok;
        } catch (LassoError $e) {
            return $e->getCode();
        } catch (Exception $e) {
            return -1;
        }
    }
    /** Profiles initiés par l'IdP */
    function sloBrws() {
        try {
            $profile = $this->getProfileObject();
            $http_method = $this->limitMethodBrws($this->identifyHttpMethod());
            $query_string = $this->getQueryString($http_method);
            return $profile->processRequestSLO($http_method, $query_string);
        } catch (LassoError $e) {
            return $e->getCode();
        } catch (Exception $e) {
            return -1;
        }
    }
    function sloSoap() {
        try { 
            $profile = $this->getProfileObject();
            return $profile->processSOAPRequestSLO();
        } catch (LassoError $e) {
            lassospkit_debuglog('Critical error: ' . $e);
            return $e->getCode();
        } catch (Exception $e) {
            return -1;
        }
    }
    /** Profiles initiés par le SP */
    function sloReturn() {
        try {
            $profile = $this->getProfileObject();
            $http_method = $this->limitMethodBrws($this->identifyHttpMethod());
            $query_string = $this->getQueryString($http_method);
            return $profile->processResponseSLO($http_method, $query_string);
        } catch (LassoError $e) {
            return $e->getCode();
        } catch (Exception $e) {
            return -1;
        }
    }
    function identifyHttpMethod() {
        if (isset($_POST) && ( isset($_POST['SAMLResponse']) || isset($_POST['SAMLRequest']))) {
            return LASSO_HTTP_METHOD_POST;
        }
        if (isset($_GET) && ( isset($_GET['SAMLResponse']) || isset($_GET['SAMLRequest']) )) {
            return LASSO_HTTP_METHOD_REDIRECT;
        }
        if (isset($_GET) && ( isset($_GET['SAMLart']))) {
            return LASSO_HTTP_METHOD_ARTIFACT_GET;
        }
        if (isset($_POST) && ( isset($_POST['SAMLart']))) {
            return LASSO_HTTP_METHOD_ARTIFACT_POST;
        }
        return LASSO_HTTP_METHOD_NONE;
    }
    function getQueryString($http_method) {
        switch ($http_method) {
            case LASSO_HTTP_METHOD_POST:
            case LASSO_HTTP_METHOD_ARTIFACT_POST:
                return @file_get_contents('php://input');
            case LASSO_HTTP_METHOD_REDIRECT:
            case LASSO_HTTP_METHOD_ARTIFACT_GET:
                return $_SERVER['QUERY_STRING'];
        }
        return null;
    }
    /** Restrict possible method for HTTP endpoints, 
     so forbid using SOAP on HTTP endpoints. */
    function limitMethodBrws($http_method) {
        switch ($http_method) {
            case LASSO_HTTP_METHOD_POST:
            case LASSO_HTTP_METHOD_ARTIFACT_POST:
            case LASSO_HTTP_METHOD_REDIRECT:
            case LASSO_HTTP_METHOD_ARTIFACT_GET:
                return $http_method;
        }
        return LASSO_HTTP_METHOD_NONE;
    }
}

